/* $LAAS: codels.h 2008/12/08 15:15:44 mallet $ */

/* 
 * Copyright (c) 2007-2008 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 *					Anthony Mallet on Mon Jun 25 2007
 */

#ifndef H_HRP2_CODELS
#define H_HRP2_CODELS

#include "server/hrp2Header.h"

#ifdef __cplusplus
# include <cstdio>
# include "HRPcontroller-idl.h"
#else
# include <stdio.h>
#endif

#include "t3d/t3d.h"

#include "genom-openhrp/genom-hrp2.h"


/** Encoders std dev. is 0.1 degrees */
#define HRP2_ENCSTDDEV	(0.1*M_PI/180.)

/** Inclinometers std dev. is 1 degree */
#define HRP2_INCSTDDEV	(1.*M_PI/180.)

/** Gyrometers std dev. is 10 degrees */
#define HRP2_GYROSTDDEV	(10.*M_PI/180.)

/** Min threshold in Newton on force sensor value to consider a foot fixed */
#define HRP2_FORCETH	(20)

/** Branch prediction */
#ifdef __GNUC__
# if __GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ >= 96)
#  define unlikely(x)	if (__builtin_expect((x), 0))
# else
#  define unlikely(x)	if (x)
# endif
#else
#  define unlikely(x)	if (x)
#endif

#ifdef __cplusplus
extern "C" {
#endif

  int		hrp2_odosetpos(const double angles[GHRP2_MAXDOF],
			const T3D *waist);
  int		hrp2_odogetpos(const double angles[GHRP2_MAXDOF],
			const double lfootF[6], const double rfootF[6],
			double imupitch, double imuroll, T3D *waist);
  int		hrp2_odolog(FILE *f);

  int		hrp2_imurpy(const double angles[GHRP2_MAXDOF],
			int len, const double imurpy[], T3D *waist);

#ifdef __cplusplus
  int		hrp2_logreadstate(FILE *log, OpenHRP::RobotState_out state);
  int		hrp2_logwritestate(FILE *log, OpenHRP::RobotState_var state);
#endif
  int		hrp2_logwritedrecord(FILE *log, const char *title,
			unsigned int elems, const double *array);
  int		hrp2_logend(FILE *log);

  STATUS	hrp2PomGetPanTiltLink(double pan, double tilt, POM_EULER_V *e);

#ifdef __cplusplus
}
#endif


#endif /* H_HRP2_CODELS */
