/* $LAAS: hrp2CntrlTaskCodels.cc 2008/07/25 15:06:47 mallet $ */

/* 
 * Copyright (c) 2008 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 *					Anthony Mallet on Fri Jul 25 2008
 */

static const char _rcsid[] =
  "$LAAS: hrp2CntrlTaskCodels.cc 2008/07/25 15:06:47 mallet $";

#include <portLib.h>
#include <math.h>

#include "server/hrp2Header.h"
#include "codels.h"


/* --- hrp2SetOdoCntrl ----------------------------------------------- */

/** \brief control codel of CONTROL request SetOdo
 *
 * Report: OK
 *                 S_hrp2_UNEXPECTED_FAILURE
 *
 * Returns:    OK or ERROR
 */

STATUS
hrp2SetOdoCntrl(GHrp2SetOdo *pos, int *report)
{
  T3D waist = t3dUBryanIdentity;

  waist.ubryan.yaw = pos->yaw;
  waist.ubryan.pitch = pos->pitch;
  waist.ubryan.roll = pos->roll;
  waist.ubryan.x = pos->x;
  waist.ubryan.y = pos->y;
  waist.ubryan.z = pos->z;

  waist.ubryan.v[0] = pos->tyaw * pos->tyaw / 9.;
  waist.ubryan.v[2] = pos->tpitch * pos->tpitch / 9.;
  waist.ubryan.v[5] = pos->troll * pos->troll / 9.;
  waist.ubryan.v[9] = pos->tx * pos->tx / 9.;
  waist.ubryan.v[14]= pos->ty * pos->ty / 9.;
  waist.ubryan.v[20]= pos->tz * pos->tz / 9.;

  if (hrp2_odosetpos(SDI_F->config.angles, &waist))
    *report = S_hrp2_UNEXPECTED_FAILURE;

  return OK;
}


/* --- hrp2GetOdoCntrl ----------------------------------------------- */

/** \brief control codel of CONTROL request GetOdo
 *
 * Report: OK
 *                 S_hrp2_UNEXPECTED_FAILURE
 *
 * Returns:    OK or ERROR
 */

STATUS
hrp2GetOdoCntrl(int *report)
{
  SDI_F->pos.yaw = SDI_F->odometry.main.euler.yaw;
  SDI_F->pos.pitch = SDI_F->odometry.main.euler.pitch;
  SDI_F->pos.roll = SDI_F->odometry.main.euler.roll;
  SDI_F->pos.x = SDI_F->odometry.main.euler.x;
  SDI_F->pos.y = SDI_F->odometry.main.euler.y;
  SDI_F->pos.z = SDI_F->odometry.main.euler.z;

  SDI_F->pos.tyaw = 3.*sqrt(SDI_F->odometry.main.var.voo);
  SDI_F->pos.tpitch = 3.*sqrt(SDI_F->odometry.main.var.vpp);
  SDI_F->pos.troll = 3.*sqrt(SDI_F->odometry.main.var.vrr);
  SDI_F->pos.tx = 3.*sqrt(SDI_F->odometry.main.var.vxx);
  SDI_F->pos.ty = 3.*sqrt(SDI_F->odometry.main.var.vyy);
  SDI_F->pos.tz = 3.*sqrt(SDI_F->odometry.main.var.vzz);

  return OK;
}
