/* $LAAS: hrp2Hrp2Codels.cc 2008/12/08 15:16:00 mallet $ */

/*
 * Copyright (c) 2007-2008 IS/AIST-ST2I/CNRS
 *	Joint Japanese-French Robotics Laboratory (JRL).
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 *					Anthony Mallet on Mon Jun 25 2007
 */

static const char _rcsid[] = "$LAAS: hrp2Hrp2Codels.cc 2008/12/08 15:16:00 mallet $";

#include <portLib.h>
#include <limits.h>
#include <err.h>

#include <iostream>

#include "server/hrp2Header.h"

#include "netshring.h"
#include "genom-openhrp-plugin-idl.h"
#include "SequencePlayer-idl.h"
#include "stateProvider-idl.h"
#include "joints.h"

#include "codels.h"

using namespace ::std;
using namespace ::genom::hrp2;


/* --- local data ---------------------------------------------------- */

static CORBA::ORB_var orb;
static GenomOpenHRP_var genom_plugin;
static stateProvider_var openHRPsp;
static SequencePlayer_var openHRPseq;

/** current robot state, exported by stateProvider plugin. */
static OpenHRP::RobotState_var robotState;

/** fifo of pending configs to send to OpenHRP. */
static GenomConfigList configs;

/** Log file handle */
static FILE *loghandle;

/** PlayBack file handle */
static FILE *playbackhandle;

/** XXX temp */
static POSTER_ID	pomPosId;


/* --- hrp2Hrp2Init -------------------------------------------------- */

/** \brief Initialization codel (fIDS, ...)
 *
 * \param[out] report	Execution status
 *
 * \return OK or ERROR
 */

STATUS
hrp2Hrp2Init(int *report)
{
  SDI_F->initialized = 0;
  SDI_F->logging = 0;

  /* initialize odometry */
  pomPosId = NULL;
  SDI_F->odometry.kind = POM_ME_ABSOLUTE;
  SDI_F->odometry.unused = 0;
  SDI_F->odometry.confidence = 0.1;
  SDI_F->odometry.date1 = 0;
  SDI_F->odometry.date2 = 0;
  memset(&SDI_F->odometry.main, 0, sizeof(SDI_F->odometry.main));
  memset(&SDI_F->odometry.VLocal, 0, sizeof(SDI_F->odometry.VLocal));

  /* Compute static pom information */
  for(int i = 0; i < HRP2_MAXLINK; i++)
    strncpy(SDI_F->pomSE[pomLinkNames[i].value].seName, pomLinkNames[i].name,
	    sizeof(SDI_F->pomSE[pomLinkNames[i].value].seName));

  /* Initialize pending configs */
  SDI_F->time = 0.;
  configs.length(0);

  fprintf(stderr, "task initialized at %gms\n", (1.0/sysClkRateGet())*1000.);
  return OK;
}


/*------------------------------------------------------------------------
 *
 * hrp2Hrp2End  --  Termination codel
 *
 * Description:
 *
 * Returns:    OK or ERROR
 */

STATUS
hrp2Hrp2End()
{
  if (SDI_F->initialized)
    switch(SDI_F->mode) {
      case HRP2_LIVE:
	orb->shutdown(1);
	break;

      case HRP2_PLAYBACK:
	break;
    }

  return OK;
}


/* --- hrp2Hrp2Perm -------------------------------------------------- */

/** \brief Codel of permanent activity
 *
 * \param[out] report		OK
 *
 * \returns OK or ERROR
 */

STATUS
hrp2Hrp2Perm(int *report)
{
  unsigned int i;
  T3D waist, imurpy;

  if (!SDI_F->initialized) return OK;

  if (!pomPosId) posterFind((char *)"pomPos", &pomPosId);
  if (pomPosId) {
    POM_POS pos;
    posterRead(pomPosId, 0, &pos, sizeof(pos));
    SDI_F->odometry.date1 = pos.pomTickDate;
  }

  switch(SDI_F->mode) {
    case HRP2_LIVE:
      try {
	CORBA::Double percent;

	/* push as many configs as possible */
	if (configs.length() > 0) {
	  CORBA::Long n = genom_plugin->push(configs, percent);
	  if (n > 0) {
	    /* copy last pushed config to robot state */
	    SDI_F->target.time = configs[n-1].time;
	    for(i = 0; i < GHRP2_MAXDOF; i++)
	      SDI_F->target.angles[jointNames[i].genom] =
		configs[n-1].angles[jointNames[i].openHRP];
	    for(i = 0; i < 3; i++) {
	      SDI_F->target.zmp[i] = configs[n-1].zmp[i];
	      SDI_F->target.waistRpy[i] = configs[n-1].waistRpy[i];
	    }

	    /* shift config buffer according to the number of config sent */
	    for(unsigned int i = n; i < configs.length(); i++) {
	      configs[i-n] = configs[i];
	    }

	    /* resize buffer */
	    configs.length(configs.length()-n);
	  }
	}

	/* get actual current state */
	openHRPsp->getActualState(robotState);
      }
      catch (CORBA::Exception& ex) {
	SDI_F->initialized = 0;
	cerr << "Disconnecting after CORBA::Exception:" << ex._name() << endl;
	return OK;
      }

      break;

    case HRP2_PLAYBACK:
      if (playbackhandle) {
	if (hrp2_logreadstate(playbackhandle, robotState)) {
	  warnx("closing playback file");
	  fclose(playbackhandle);
	  playbackhandle = NULL;
	}
      } else
	/* if no playback file, just do nothing and return */
	return OK;

      break;
  }

  /* copy current state to poster */
  for(i = 0; i < GHRP2_MAXDOF; i++)
    SDI_F->config.angles[jointNames[i].genom] =
      robotState->angle[jointNames[i].openHRP];

  /* compute pom SEs */
  hrp2PomGetPanTiltLink(SDI_F->config.angles[GHRP2_HEAD_JOINT0],
			SDI_F->config.angles[GHRP2_HEAD_JOINT1],
			&SDI_F->pomSE[HRP2_HEAD_TO_NECK].seConfig);

  hrp2PomGetPanTiltLink(SDI_F->config.angles[GHRP2_CHEST_JOINT0],
			SDI_F->config.angles[GHRP2_CHEST_JOINT1],
			&SDI_F->pomSE[HRP2_CHEST_TO_WAIST].seConfig);

  /* compute imu RPY */
  if (hrp2_imurpy(SDI_F->config.angles, robotState->attitude[0].length(),
		  &robotState->attitude[0][0], &imurpy)) {
    warnx("error while computing waist RPY");
    t3dInit(&imurpy, T3D_U_BRYAN, T3D_ALLOW_CONVERSION);
  }
  t3dConvertTo(T3D_U_BRYAN, &imurpy);

  /* compute odometry */
  if (!hrp2_odogetpos(SDI_F->config.angles,
		      &robotState->force[0][0], &robotState->force[1][0],
		      imurpy.ubryan.pitch, imurpy.ubryan.roll,
		      &waist)) {
    t3dConvertTo(T3D_U_BRYAN, &waist);

    SDI_F->odometry.main.euler.yaw = waist.ubryan.yaw;
    SDI_F->odometry.main.euler.pitch = waist.ubryan.pitch;
    SDI_F->odometry.main.euler.roll = waist.ubryan.roll;
    SDI_F->odometry.main.euler.x = waist.ubryan.x;
    SDI_F->odometry.main.euler.y = waist.ubryan.y;
    SDI_F->odometry.main.euler.z = waist.ubryan.z;

    memcpy(&SDI_F->odometry.main.var,
	   waist.ubryan.v, sizeof(SDI_F->odometry.main.var));

    /* XXX set RPY in the state poster with odometry */
    SDI_F->config.waistRpy[2] = waist.ubryan.yaw;
    SDI_F->config.waistRpy[1] = waist.ubryan.pitch;
    SDI_F->config.waistRpy[0] = waist.ubryan.roll;

  } else warnx("error while computing odometry");

  /* log data */
  if (loghandle) {
    hrp2_logwritestate(loghandle, robotState);

    fprintf(loghandle,"waist: ");
    for(i=0; i<6; i++) fprintf(loghandle, "%12g ", waist.ubryan.bryan[i]);
    fprintf(loghandle, "\n");

    hrp2_odolog(loghandle);
    hrp2_logend(loghandle);
  }

  return OK;
}


/* --- hrp2InitExec -------------------------------------------------- */

/** \brief codel EXEC of Init
 *
 * \param[out] report		OK
 *				S_hrp2_OPENHRP_NOT_FOUND
 *
 * \return EXEC END ETHER FAIL ZOMBIE
 */

ACTIVITY_EVENT
hrp2InitExec(GHrp2Mode *mode, int *report)
{
  switch(*mode) {

    case HRP2_LIVE: {
      CORBA::Object_var obj;
      CosNaming::NamingContext_var root;
      OpenHRP::PluginManager_var pm;
      int i, j;

      /* connect to CORBA services */

      try {
	int argc = 0;
	orb = CORBA::ORB_init(argc, NULL);

	obj = orb->resolve_initial_references("NameService");
	root = CosNaming::NamingContext::_narrow(obj);
	if( CORBA::is_nil(root) ) {
	  cerr << "Failed to narrow the root naming context." << endl;
	  *report = S_hrp2_NAMESERVER_NOT_FOUND;
	  return ETHER;
	}
      }
      catch (CORBA::NO_RESOURCES&) {
	cerr << "Caught NO_RESOURCES exception. You must configure omniORB "
	     << "with the location" << endl
	     << "of the naming service." << endl;
	*report = S_hrp2_NAMESERVER_NOT_FOUND;
	return ETHER;
      }
      catch(CORBA::Exception& ex) {
	cerr << "Caught CORBA::Exception:" << ex._name() << endl;
	*report = S_hrp2_CORBA_NOT_FOUND;
	return ETHER;
      }

      /* Locate necessary OpenHRP plugins */

      try {
	CosNaming::Name name;

	// Plugin manager
	name.length(1);
	name[0].id = CORBA::string_dup("motionsys");
	pm = OpenHRP::PluginManager::_narrow(root->resolve(name));
	cout << "Found OpenHRP plugin manager" << endl;

	// State Provider
	pm->load("provider");
	pm->create("provider", "sp", "");

	name.length(1);
	name[0].id = CORBA::string_dup("sp");
	openHRPsp = stateProvider::_narrow(root->resolve(name));
	openHRPsp->start();
	cout << "Started OpenHRP state provider" << endl;

	// Genom plugin
	name.length(1);
	name[0].id = CORBA::string_dup("genom-openhrp");
	genom_plugin = GenomOpenHRP::_narrow(root->resolve(name));
	genom_plugin->start();
	cout << "Started genom plugin" << endl;

	// Seq. plugin
	pm->load("seqplay");
	pm->create("seqplay", "seq", "");
	cout << "Loaded OpenHRP seq. plugin" << endl;

	name.length(1);
	name[0].id = CORBA::string_dup("seq");
	obj = root->resolve(name);
	openHRPseq = SequencePlayer::_narrow(obj);
	openHRPseq->start();
	cout << "Started OpenHRP seq. plugin" << endl;
      }

      catch(omniORB::fatalException& fe) {
	cerr << "Caught omniORB::fatalException:" << endl;
	cerr << " file: " << fe.file() << endl;
	cerr << " line: " << fe.line() << endl;
	cerr << " mesg: " << fe.errmsg() << endl;

	*report = S_hrp2_OPENHRP_NOT_FOUND;
	return ETHER;
      }

      catch(CORBA::Exception& ex) {
	cerr << "Caught CORBA::Exception:" << ex._name() << endl;
	*report = S_hrp2_OPENHRP_NOT_FOUND;
	return ETHER;
      }

      /* get initial target config */
      try {
	openHRPsp->getReferenceState(robotState);
      }
      catch (CORBA::Exception& ex) {
	SDI_F->initialized = 0;
	cerr << "Disconnecting after CORBA::Exception:" << ex._name() << endl;
	return ETHER;
      }

      /* copy current target to poster */
      SDI_F->target.time = 0.;
      for(i = 0; i < GHRP2_MAXDOF; i++)
	SDI_F->target.angles[jointNames[i].genom] =
	  robotState->angle[jointNames[i].openHRP];
      for(i = 0; i < 3; i++)
	SDI_F->target.zmp[i] = robotState->zmp[i];

      T3D rpy(T3D_MATRIX, T3D_ALLOW_CONVERSION);
      for(i = 0; i < 3; i++) for(j = 0; j < 3; j++)
	rpy.matrix(i,j) = robotState->baseAtt[i*3+j];
      rpy.convertTo(T3D_BRYAN);
      for(i = 0; i < 3; i++)
	SDI_F->target.waistRpy[i] = rpy.bryan[2-i];

      break;
    }

    case HRP2_PLAYBACK:
      /* playback mode doesn't need any initialization */
      break;
  }

  SDI_F->initialized = 1;
  warnx("module initialized");
  return ETHER;
}


/* --- SetJointAngle ------------------------------------------------- */

/** \brief codel START of SetJointAngle
 *
 * \param[in] setJointAngle	requests parameters
 * \param[out] report		OK
 *				S_hrp2_NOT_INITIALIZED
 *
 * \return EXEC END ETHER FAIL ZOMBIE
 */

ACTIVITY_EVENT
hrp2SetJointAngleStart(GHrp2SetJointAngle *setJointAngle, int *report)
{
  unsigned long i, n;

  if (!SDI_F->initialized) {
    *report = S_hrp2_NOT_INITIALIZED;
    return ETHER;
  }

  n = configs.length();
  configs.length(n+2);
  configs[n].angles.length(GHRP2_MAXDOF);
  configs[n].zmp.length(3);
  configs[n].waistRpy.length(3);

  /* get current target */
  configs[n].time = SDI_F->target.time + GHRP2_PLUGIN_PERIOD;
  configs[n].stable = 0;
  for(i = 0; i<GHRP2_MAXDOF; i++)
    configs[n].angles[jointNames[i].openHRP] =
      SDI_F->target.angles[jointNames[i].genom];
  for(i = 0; i<3; i++) {
    configs[n].zmp[i] = SDI_F->target.zmp[i];
    configs[n].waistRpy[i] = SDI_F->target.waistRpy[i];
  }

  /* update the selected angle */
  configs[n+1] = configs[n];
  configs[n+1].time += setJointAngle->delay;
  configs[n+1].stable = 1;
  configs[n+1].angles[jointNames[setJointAngle->joint].openHRP] =
    setJointAngle->angle;

  printf("%g\n", configs[n+1].time);
  return EXEC;
}

/** \brief codel EXEC of SetJointAngle
 *
 * \param[in] setJointAngle	requests parameters
 * \param[out] report		OK
 *				S_hrp2_NOT_INITIALIZED
 *
 * \return EXEC END ETHER FAIL ZOMBIE
 */

ACTIVITY_EVENT
hrp2SetJointAngleExec(GHrp2SetJointAngle *setJointAngle, int *report)
{
  if (configs.length() > 0) return EXEC;
  return ETHER;
}


/* --- GetLinkPos ---------------------------------------------------- */

/** \brief codel EXEC of GetLinkPos
 *
 * \param[in] joint		joint name
 * \param[out] getLinkPos	configuration
 * \param[out] report		OK
 *				S_hrp2_NOT_INITIALIZED
 *				S_hrp2_BAD_CONFIGURATION
 *
 * \return EXEC END ETHER FAIL ZOMBIE
 */

ACTIVITY_EVENT
hrp2GetLinkPosMain(ghrp2_joint_index_t *joint, GHrp2LinkPos *getLinkPos, int *report)
{
#if 0
  int i, j;
  T3D t3d;
  CORBA::Boolean s;
  Configuration c;

  if (!SDI_F->initialized) {
    *report = S_hrp2_NOT_INITIALIZED;
    return ETHER;
  }

  /* set model configuration to match actual state of the robot */
  openHRPdyn->setModelJoints(robotState->angle);

  /* get link configuration */
  s = openHRPdyn->getLinkAbsConfiguration(jointNames[*joint].linkname, c);
  if (!s) {
    *report = S_hrp2_NOT_INITIALIZED;
  }

  /* compute euler values */
  t3d.matrix = t3dMatrixIdentity;
  for(i=0;i<3;i++) {
    for(j=0;j<3;j++) t3d.matrix.matrix[j +i*4] = c.rot[j +i*3];
    t3d.matrix.matrix[3 +i*4] = c.trs[0 +i];
  }
  t3dConvertTo(T3D_EULER, &t3d);

  for(i=0;i<3;i++) {
    getLinkPos->t[i] = t3d.euler.euler[3+i];
    getLinkPos->r[i] = t3d.euler.euler[i];
  }
#endif

  return ETHER;
}


/* --- SetLinkPos ---------------------------------------------------- */

/** \brief codel START of SetLinkPos
 *
 * \param[in] setLinkPos	requests parameters
 * \param[out] report		OK
 *				S_hrp2_NOT_INITIALIZED
 *
 * \return EXEC END ETHER FAIL ZOMBIE
 */

ACTIVITY_EVENT
hrp2SetLinkPosStart(GHrp2SetLinkPos *setLinkPos, int *report)
{
#if 0
  int i,j;
  T3D t3d;
  CORBA::Boolean s;
  dsequence_var ref;
  Configuration c;

  if (!SDI_F->initialized) {
    *report = S_hrp2_NOT_INITIALIZED;
    return ETHER;
  }

  /* convert euler to matrix */
  t3d.euler = t3dEulerIdentity;
  for(i=0;i<3;i++) {
    t3d.euler.euler[i] = setLinkPos->pos.r[i];
    t3d.euler.euler[3+i] = setLinkPos->pos.t[i];
  }
  t3dConvertTo(T3D_MATRIX, &t3d);

  for(i=0;i<3;i++) {
    for(j=0;j<3;j++) c.rot[j +i*3] = t3d.matrix.matrix[j +i*4];
    c.trs[0 +i] = t3d.matrix.matrix[3 +i*4];
  }

  /* compute configuration */
  s = openHRPdyn->moveModelAbs(jointNames[setLinkPos->start].linkname,
			       jointNames[setLinkPos->end].linkname,
			       c);
  if (!s) {
    *report = S_hrp2_BAD_CONFIGURATION;
    return ETHER;
  }

  /* get configuration */
  openHRPdyn->getModelJoints(ref);

  /* send configuration to sequence player plugin */
  openHRPseq->setJointAngles(ref, setLinkPos->delay);
#endif

  return EXEC;
}


/** \brief codel EXEC of SetLinkPos
 *
 * \param[in] setLinkPos	requests parameters
 * \param[out] report		OK
 *				S_hrp2_NOT_INITIALIZED
 *
 * \return EXEC END ETHER FAIL ZOMBIE
 */

ACTIVITY_EVENT
hrp2SetLinkPosMain(GHrp2SetLinkPos *setLinkPos, int *report)
{
  if (openHRPseq->isEmpty()) return ETHER;
  return EXEC;
}


/* --- GoHalfSitting ------------------------------------------------- */

/** \brief codel EXEC of GoHalfSitting
 *
 * \param[out] report		OK
 *				S_hrp2_NOT_INITIALIZED
 *
 * \return EXEC
 */

ACTIVITY_EVENT
hrp2GoHalfSittingMain(int *report)
{
  if (!SDI_F->initialized) {
    *report = S_hrp2_NOT_INITIALIZED;
    return ETHER;
  }

  openHRPseq->goHalfSitting(0);
  return ETHER;
}


/* --- Play ---------------------------------------------------------- */

/** Read a trajectory file and send it to the HRP-2 controller
 */

static FILE *filepos, *filezmp, *filerpy;
static int fileindex;
static double timeoffset;

ACTIVITY_EVENT
hrp2PlayStart(GHrp2Play *play, int *report)
{
  char path[PATH_MAX];
  char file[PATH_MAX];

#ifdef GHRP2_PREFIX
  if (play->base[0] != '/') {
    snprintf(path, sizeof(path)-1, GHRP2_PREFIX "/%s", play->base);
  } else
#endif
    strncpy(path, play->base, sizeof(path)-1);

  snprintf(file, sizeof(file)-1, "%s%s", path, play->joint);
  filepos = fopen(file, "r");
  if (!filepos) { *report = S_hrp2_NO_SUCH_FILE; return ETHER; }

  snprintf(file, sizeof(file)-1, "%s%s", path, play->zmp);
  filezmp = fopen(file, "r");
  if (!filezmp) { *report = S_hrp2_NO_SUCH_FILE; return ETHER; }

  snprintf(file, sizeof(file)-1, "%s%s", path, play->rpy);
  filerpy = fopen(file, "r");
  if (!filerpy) { *report = S_hrp2_NO_SUCH_FILE; return ETHER; }

  warnx("playing file `%s'\n", path);

  if (configs.length() > 0) {
    timeoffset = configs[configs.length()-1].time;
  } else
    timeoffset = 0;

  return EXEC;
}


ACTIVITY_EVENT
hrp2PlayMain(GHrp2Play *play, int *report)
{
  unsigned long prev, n;
  ghrp2_config_t target;
  double time;
  int i;

  /* flood fill the buffer */
  prev = configs.length();
  if (configs.length() > 20) return EXEC;

  do {
    fileindex++;

    /* read angles */
    i = fscanf(filepos, "%lf", &target.time);
    if (i == EOF) return END;

    if (i != 1) {
      warnx("bad line in pos at index %d", fileindex);
      *report = S_hrp2_BAD_FILE;
      return END;
    }
    for(i = 0; i<GHRP2_MAXDOF; i++)
      if (fscanf(filepos, "%lf", &target.angles[i]) != 1) {
	warnx("bad line in pos at index %d, column %d", fileindex, i);
	*report = S_hrp2_BAD_FILE;
	return END;
      }

    /* read zmp */
    if (fscanf(filezmp, "%lf", &time) != 1 || time != target.time) {
      warnx("bad time in zmp at index %d", fileindex);
      *report = S_hrp2_BAD_FILE;
      return END;
    }
    for(i = 0; i<3; i++)
      if (fscanf(filezmp, "%lf", &target.zmp[i]) != 1) {
	warnx("bad line in zmp at index %d, column %d", fileindex, i);
	*report = S_hrp2_BAD_FILE;
	return END;
      }

    /* read rpy */
    if (fscanf(filerpy, "%lf", &time) != 1 || time != target.time) {
      warnx("bad time in rpy at index %d", fileindex);
      *report = S_hrp2_BAD_FILE;
      return END;
    }
    for(i = 0; i<3; i++)
      if (fscanf(filerpy, "%lf", &target.waistRpy[i]) != 1) {
	warnx("bad line in zmp at index %d, column %d", fileindex, i);
	*report = S_hrp2_BAD_FILE;
	return END;
      }

    /* send data */
    target.time += timeoffset;

    n = configs.length();
    configs.length(n+1);
    configs[n].angles.length(GHRP2_MAXDOF);
    configs[n].zmp.length(3);
    configs[n].waistRpy.length(3);

    configs[n].time = target.time;
    configs[n].stable = 0;
    for(i = 0; i<GHRP2_MAXDOF; i++) configs[n].angles[i] = target.angles[i];
    for(i = 0; i<3; i++) configs[n].zmp[i] = target.zmp[i];
    for(i = 0; i<3; i++) configs[n].waistRpy[i] = target.waistRpy[i];

    if (configs.length() > 20) break;
  } while(1);

  if (configs.length() != prev)
    printf("read %lu samples\n", configs.length() - prev);
  return EXEC;
}


ACTIVITY_EVENT
hrp2PlayEnd(GHrp2Play *play, int *report)
{
  if (configs.length() != 0) return END;

  if (filepos) { fclose(filepos); filepos = NULL; }
  if (filezmp) { fclose(filezmp); filepos = NULL; }
  if (filerpy) { fclose(filerpy); filepos = NULL; }

  return ETHER;
}


/* --- Track --------------------------------------------------------- */

/** hrp2TrackStart  -  codel START of Track
 *
 * Reports:      OK
 *              S_hrp2_NO_FEED
 *
 * \return START EXEC END ETHER FAIL ZOMBIE
 */

static netshring_id_t track_source;

ACTIVITY_EVENT
hrp2TrackStart(GHrp2Track *track, int *report)
{
  track_source = netshring_connect(track->reference, SHRNG_SYNC);
  if (!track_source) {
    fprintf(stderr, "cannot find feed `%s'\n", track->reference);
    *report = S_hrp2_NO_FEED;
    return ETHER;
  }

  fprintf(stdout, "tracking from feed `%s'\n", track->reference);
  return EXEC;
}


/** hrp2TrackMain  -  codel EXEC of Track
 *
 * Reports:      OK
 *              S_hrp2_NO_FEED
 *              S_hrp2_FEED_UNREADABLE
 *              S_hrp2_CANNOT_PUSH
 *
 * \return START EXEC END ETHER FAIL ZOMBIE
 */

ACTIVITY_EVENT
hrp2TrackMain(GHrp2Track *track, int *report)
{
  unsigned long prev;
  int i, n;
  ghrp2_config_t target;

  /* flood fill the buffer */
  prev = configs.length();
  if (configs.length() > 20) return EXEC;

  do {
    n = netshring_pop(track_source, &target, sizeof(target), SHRNG_ASYNC);
    if (n == 0) break;
    if (n < 0 || n != sizeof(target)) {
      *report = S_hrp2_FEED_UNREADABLE; return END;
    }

    if (target.time == -1.0) {
      fprintf(stdout, "end of motion from feed `%s'\n", track->reference);
      return END;
    }

    n = configs.length();
    configs.length(n+1);
    configs[n].angles.length(GHRP2_MAXDOF);
    configs[n].zmp.length(3);
    configs[n].waistRpy.length(3);

    configs[n].time = target.time;
    configs[n].stable = 0;
    for(i = 0; i<GHRP2_MAXDOF; i++) configs[n].angles[i] = target.angles[i];
    for(i = 0; i<3; i++) configs[n].zmp[i] = target.zmp[i];
    for(i = 0; i<3; i++) configs[n].waistRpy[i] = target.waistRpy[i];

    if (configs.length() > 20) break;
  } while(1);

  if (configs.length() != prev)
    printf("read %lu samples\n", configs.length() - prev);
  return EXEC;
}


/** hrp2TrackEnd  -  codel END of Track
 *
 * Reports:      OK
 *              S_hrp2_NO_FEED
 *
 * \return START EXEC END ETHER FAIL ZOMBIE
 */

ACTIVITY_EVENT
hrp2TrackEnd(GHrp2Track *track, int *report)
{
  /* disconnect source */
  if (track_source) netshring_disconnect(track_source);
  track_source = NULL;

  return ETHER;
}


/** hrp2TrackInter  -  codel INTER of Track
 *
 * Reports:      OK
 *              S_hrp2_NO_FEED
 *
 * \return START EXEC END ETHER FAIL ZOMBIE
 */

ACTIVITY_EVENT
hrp2TrackInter(GHrp2Track *track, int *report)
{
  return hrp2TrackEnd(track, report);
}


/* --- TrackDebug ---------------------------------------------------- */

/** hrp2TrackDebugMain  -  codel EXEC of TrackDebug
 *
 * Reports:      OK
 *
 * \return ETHER
 */

ACTIVITY_EVENT
hrp2TrackDebugMain(int *report)
{
  genom_plugin->debug();
  return ETHER;
}


/* --- Log ----------------------------------------------------------- */

/** hrp2LogMain  -  codel MAIN of Log
 *
 * Reports:      OK
 *              S_hrp2_NO_FILE
 *
 * \return START EXEC END ETHER FAIL ZOMBIE
 */

ACTIVITY_EVENT
hrp2LogMain(GHrp2Log *log, int *report)
{
  if (loghandle) {
    SDI_F->logging = 0;
    fclose(loghandle);
    loghandle = NULL;
  }
  if (log->file[0] == '\0') return ETHER;

  loghandle = fopen(log->file, "w");
  if (!loghandle) {
    *report = S_hrp2_NO_FILE;
    return ETHER;
  }

  SDI_F->logging = 1;
  return ETHER;
}


/* --- PlayBack ------------------------------------------------------ */

/** hrp2LogMain  -  codel MAIN of Log
 *
 * Reports:      OK
 *              S_hrp2_BAD_INIT_MODE
 *              S_hrp2_NO_FILE
 *
 * \return START EXEC END ETHER FAIL ZOMBIE
 */

ACTIVITY_EVENT
hrp2PlayBackMain(GHrp2Log *log, int *report)
{
  if (SDI_F->mode != HRP2_PLAYBACK) {
    *report = S_hrp2_BAD_INIT_MODE;
    return ETHER;
  }

  if (playbackhandle) {
    fclose(playbackhandle);
    playbackhandle = NULL;
  }
  if (log->file[0] == '\0') return ETHER;

  playbackhandle = fopen(log->file, "r");
  if (!playbackhandle) {
    *report = S_hrp2_NO_FILE;
    return ETHER;
  }

  return ETHER;
}
