/* $LAAS: imu.c 2008/12/08 15:23:17 mallet $ */

/*
 * Copyright (c) 2008 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 *					Anthony Mallet on Thu Jul 31 2008
 */

static const char _rcsid[] = "$LAAS: imu.c 2008/12/08 15:23:17 mallet $";

#include <math.h>

#include "t3d/t3d.h"
#include "genom-openhrp/genom-hrp2.h"
#include "codels.h"


/* --- local data ---------------------------------------------------- */


/* --- hrp2_imurpy --------------------------------------------------- */

/** \brief Get roll pitch and yaw of the waist from IMU data
 *
 * \param[in] f		Log file descriptor
 *
 * \return 0 on success
 */

int
hrp2_imurpy(const double angles[GHRP2_MAXDOF], int len, const double imurpy[],
	    T3D *waist)
{
  T3D chest, imu;

  if (!t3dInit(waist, T3D_U_BRYAN, T3D_ALLOW_CONVERSION)) return -1;

  /* imurpy is a rotation matrix in simulation and a quaternion in real
     mode */
  if (!t3dInit(&imu, T3D_MATRIX, T3D_ALLOW_CONVERSION)) return -1;
  if (len == 4) {
    double qw = imurpy[0];
    double qx = imurpy[1];
    double qy = imurpy[2];
    double qz = imurpy[3];
    double qxqy = qx * qy;
    double qxqw = qx * qw;
    double qxqz = qx * qz;
    double qyqz = qy * qz;
    double qyqw = qy * qw;
    double qzqw = qz * qw;
    double qx2 = qx * qx;
    double qy2 = qy * qy;
    double qz2 = qz * qz;

    imu.matrix.matrix[0] = 1 - 2*qy2 - 2*qz2;
    imu.matrix.matrix[1] = 2*qxqy - 2*qzqw;
    imu.matrix.matrix[2] = 2*qxqz + 2*qyqw;

    imu.matrix.matrix[4] = 2*qxqy + 2*qzqw;
    imu.matrix.matrix[5] = 1 - 2*qx2 - 2*qz2;
    imu.matrix.matrix[6] = 2*qyqz - 2*qxqw;

    imu.matrix.matrix[8] = 2*qxqz - 2*qyqw;
    imu.matrix.matrix[9] = 2*qyqz + 2*qxqw;
    imu.matrix.matrix[10] = 1 - 2*qx2 - 2*qy2;
  } else {
    imu.matrix.matrix[0] = imurpy[0];
    imu.matrix.matrix[1] = imurpy[1];
    imu.matrix.matrix[2] = imurpy[2];

    imu.matrix.matrix[4] = imurpy[3];
    imu.matrix.matrix[5] = imurpy[4];
    imu.matrix.matrix[6] = imurpy[5];

    imu.matrix.matrix[8] = imurpy[6];
    imu.matrix.matrix[9] = imurpy[7];
    imu.matrix.matrix[10] = imurpy[8];
  }
  if (!t3dConvertTo(T3D_BRYAN, &imu)) return -1;

  waist->ubryan.yaw = angles[GHRP2_CHEST_JOINT0];
  waist->ubryan.pitch = angles[GHRP2_CHEST_JOINT1];
  waist->ubryan.v[0] = HRP2_ENCSTDDEV * HRP2_ENCSTDDEV;
  waist->ubryan.v[2] = HRP2_ENCSTDDEV * HRP2_ENCSTDDEV;
  if (!t3dInvert(waist)) return -1;

  t3dInit(&chest, T3D_U_BRYAN, T3D_ALLOW_CONVERSION);
  chest.ubryan.yaw = imu.bryan.yaw;
  chest.ubryan.pitch = imu.bryan.pitch;
  chest.ubryan.roll = imu.bryan.roll;
  chest.ubryan.v[0] = HRP2_GYROSTDDEV * HRP2_GYROSTDDEV;
  chest.ubryan.v[2] = HRP2_INCSTDDEV * HRP2_INCSTDDEV;
  chest.ubryan.v[5] = HRP2_INCSTDDEV * HRP2_INCSTDDEV;

  if (!t3dComp(waist, &chest)) return -1;

  return 0;
}
